<?php

class OcList {

    private static $SLUG_SETTINGS = 'oclist_settings';
    public static $OPTIONS_NAME = 'OCLIST_OPTIONS';
    private $options;
    private $oc;

    private $default_options = [
        'list_query' => '*:*',
        'published_property' => '',
        'article_relation_property' => '',
        'notifier_bindings' => [],
        'notifier_registered' => false,
        'notifier_url' => ''
    ];

    public function __construct() {

        if ( isset( $_POST['oclist_settings_form_posted'] ) ) {
            $this->save_settings();
        }

        $this->oc = OpenContent::getInstance();
        $this->options = get_option( self::$OPTIONS_NAME );
        add_action( 'admin_menu', array( &$this, 'admin_menu' ) );
        add_action( 'ew_notifier_update_list_properties', function($properties) {

            $options = $this->get_options();
            if(!isset($options['article_relation_property']) || $options['article_relation_property'] === '') {
                return $properties;
            }

            $relation_property = $options['article_relation_property'];
            $article_props = self::get_article_props();
            $list_properties = [
                'uuid',
                'Name',
                $relation_property . '.uuid',
                $relation_property . '.contenttype',
                $relation_property . '.' . $options['published_property']
            ];

            $list_properties = array_merge($list_properties, $article_props);

            return array_unique(array_merge($properties, $list_properties));
        } );

        add_action( 'ew_notifier_update_list_filter', function($filter) {

            $options = $this->get_options();
            if(!isset($options['article_relation_property'], $options['published_property']) || $options['article_relation_property'] === '' || $options['published_property'] === '') {
                return $filter;
            }

            $relation_property = $options['article_relation_property'];
            return $relation_property . '(q=' . $options['published_property'] . ':true|start=0|limit=100)';
        } );

        add_action('ew_notifier_oclist_added', function($bindings) {

            $options = $this->get_options();
            $options['notifier_registered'] = true;
            $options['notifier_bindings'] = isset($bindings) && $bindings !== '' ? $bindings : [];

            update_option( self::$OPTIONS_NAME, json_encode( $options ) );
        });
    }

    private function get_options() {

        if(!isset($this->oc)) {
            $this->oc = OpenContent::getInstance();
        }

        $options = get_option(self::$OPTIONS_NAME);
        $options =  $options !== false ? json_decode( $options, true ) : [];
        $options = array_replace_recursive($this->default_options, $options);
        $options['notifier_registered'] = $this->oc->getNotifierRegistered($options['notifier_url'], 'oclist');
        return $options;
    }

    public function admin_menu() {
        add_submenu_page( 'edit.php?post_type=everyboard', 'OC List Settings', __( 'OC List Settings', 'everyboard' ), 'manage_options', self::$SLUG_SETTINGS, [&$this, 'settings_page']);
    }

    public function settings_page() {

        $properties = self::get_properties('Article', null);
        $bool_properties  = self::get_properties('Article', 'BOOLEAN');
        $article_relation_properties  = self::get_properties('List', 'Article');

        $options = $this->get_options();
        include dirname( __FILE__ ) . '/views/settings.php';
    }

    public function save_settings() {

        $list_query = isset($_POST['list_query']) ? $_POST['list_query'] : '*:*';
        $published_property = isset($_POST['published_property']) ? $_POST['published_property'] : '';
        $article_relation_property = isset($_POST['article_relation_property']) ? $_POST['article_relation_property'] : '';
        $notifier_url = isset($_POST['notifier_url']) ? $_POST['notifier_url'] : '*:*';
        $options = $this->get_options();

        $settings = [
            'list_query' => $list_query,
            'published_property' => $published_property,
            'article_relation_property' => $article_relation_property,
            'notifier_url' => $notifier_url,
        ];

        $settings = array_replace_recursive($options, $settings);

        update_option( self::$OPTIONS_NAME, json_encode( $settings ) );
    }

    /**
     * Fetch property names from OC, filter by contenttype and data type.
     *
     * @param $contenttype
     * @param $type
     *
     * @return array
     */
    private static function get_properties($contenttype, $type) {

        $oc_api = new OcAPI();
        $return_array = [];

        $cache_key = 'ew_' . $contenttype . '_' . $type . '_properties';
        if(false === ($return_array = get_transient($cache_key))) {

            $url = $oc_api->getOcBaseUrl() . 'contenttypes';
            $api_response = wp_remote_get($url, $oc_api->getRemoteArgs());

            if(!is_wp_error($api_response)) {
                $http_status = wp_remote_retrieve_response_code($api_response);

                if ($http_status == "200") {
                    $json_content = json_decode(wp_remote_retrieve_body($api_response));

                    foreach($json_content->contentTypes as $content_type) {

                        if($content_type->name === $contenttype) {
                            foreach($content_type->properties as $property) {

                                if($type === null || $property->type === $type) {
                                    $return_array[] = $property->name;
                                }
                            }

                        }
                    }
                }
            }

            set_transient($cache_key, $return_array, $oc_api->getCacheTTL());
        }

        return $return_array;
    }

    public static function get_all_lists() {

        $options = get_option(self::$OPTIONS_NAME);
        $options = $options !== false ? json_decode( $options, true ) : [
            'list_query' => '*:*'
        ];

        $oc_api = new OcAPI();
        $result = $oc_api->search([
            'q' => isset($options['list_query']) ? $options['list_query'] : '*:*',
            'contenttypes' => ['List'],
            'start' => 0,
            'limit' => 100,
            'properties' => [
                'uuid',
                'Name',
                'Type'
            ]
        ]);

        $lists = [];
        foreach($result as $list) {

            if($list instanceof OcObject && isset($list->uuid[0], $list->name[0])) {
                $lists[$list->uuid[0]] = $list->name[0];
            }
        }

        return $lists;
    }

    /**
     * Function for fetching an article from a specific position from a specific list
     * @param $listId
     * @param int $positionId
     *
     * @return null
     */
    public static function get_article_from_list( $listId, $positionId = 0 ) {

        if ( $positionId === 0 )
            return null;

        $ret = null;
        $articles = self::get_approved_list_articles($listId);

        if(isset( $articles[$positionId - 1])) {
            $ret = $articles[$positionId - 1];
        }

        return $ret;
    }

    /**
     * Function for fetching one or more articles from a list, begins fetching at $positionId
     * @param $listId
     * @param int $positionId
     * @param int $limit
     *
     * @return null|array
     */
    public static function get_articles_from_list( $listId, $positionId = 0, $limit = 1 ) {

        if ( $positionId === 0 )
            return null;

        $ret = [];
        $articles = self::get_approved_list_articles($listId);

        if ( $limit > 1 ) {

            for ( $i = 0; $i < $limit; $i++ ) {
                if ( isset( $articles[( $positionId - 1 ) + $i] ) ) {
                    $ret[] = $articles[( $positionId - 1 ) + $i];
                }
            }
        } else if ( isset( $articles[$positionId - 1] ) ) {
            $ret[] = $articles[$positionId - 1];
        }

        return count( $ret ) > 0 ? $ret : null;
    }

    /**
     * Function for fetching all approved articles included in a specific list
     * Filters out articles that don't have correct status or are in the future
     * @param $listId
     *
     * @return array
     */
    public static function get_approved_list_articles( $listId ) {

        $options = get_option(self::$OPTIONS_NAME);
        $options = $options !== false ? json_decode( $options, true ) : [
            'published_property' => '',
            'article_relation_property' => ''
        ];

        if(!isset($options['article_relation_property']) || $options['article_relation_property'] === '') {
            return [];
        }

        $relation_property = $options['article_relation_property'];

        $article_props = self::get_article_props();
        $properties = [
            'uuid',
            'Name',
            $relation_property . '.uuid',
            $relation_property . '.contenttype',
            $relation_property . '.' . $options['published_property']
        ];

        $properties = array_merge($properties, $article_props);

        $oc_api = new OcAPI();

        $result = $oc_api->search([
            'q' => 'uuid:' . $listId,
            'contenttypes' => ['List'],
            'start' => 0,
            'limit' => 1,
            'properties' => $properties,
            'filters' => $relation_property . '(q=' . $options['published_property'] . ':true|start=0|limit=100)'
        ], true, false);

        $articles = [];
        if( isset( $result[0], $result[0]->uuid[0], $result[0]->name[0] ) && $result[0] instanceof OcObject ) {

            $lower_relation_property = strtolower($relation_property);
            if(isset($result[0]->{$lower_relation_property})) {
                $articles = $result[0]->{$lower_relation_property};
            }
        }

        return $articles;
    }

    public static function get_article_props() {

        $property_map = get_option('prop_map');
        $options = get_option(self::$OPTIONS_NAME);
        $options = $options !== false ? json_decode( $options, true ) : [
            'published_property' => '',
            'article_relation_property' => ''
        ];

        $article_properties = self::get_properties('Article', null);
        $properties = [];

        if(isset($property_map['Headline']) && $property_map['Headline'] !== 'unknown') {

            foreach($article_properties as $prop) {

                if(strtolower($prop) === $property_map['Headline']) {
                    $properties[] = $options['article_relation_property'] . '.' . $prop;
                    break;
                }
            }
        }

        if(isset($property_map['Section']) && $property_map['Section'] !== 'unknown') {

            foreach($article_properties as $prop) {

                if(strtolower($prop) === $property_map['Section']) {
                    $properties[] = $options['article_relation_property'] . '.' . $prop;
                    break;
                }
            }
        }

        return $properties;
    }

    public static function get_list_name_by_id( $listId ) {

        $name = 'Cant find list name';
        $oc_api = new OcAPI();
        $result = $oc_api->search([
            'q' => 'uuid:' . $listId,
            'contenttypes' => ['List'],
            'start' => 0,
            'limit' => 1,
            'properties' => [
                'uuid',
                'Name'
            ]
        ]);

        foreach($result as $list) {
            if($list instanceof OcObject && isset($list->uuid[0], $list->name[0])) {
                $name = $list->name[0];
            }
        }

        return $name;
    }

    public static function get_wp_sites() {

        $sites = get_sites([
            'site__not_in' => get_current_blog_id()
        ]);

        $wp_sites = [];
        foreach($sites as $site) {
            $site_id = (int)$site->blog_id;
            $wp_sites[$site_id] = $site->domain;
        }

        return $wp_sites;
    }
}

add_action( 'init', function () {
    new OcList();
} );